'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _graphql = require('graphql');

var _quickGraphql = require('../quick-graphql');

var _promises = require('../utils/promises');

var _actions = require('../redux/actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const THROTTLE_DELAY = 50;

const query = (mapPropsToQuery, mapPropsToVariables = () => ({})) => WrappedComponent => {
	var _class, _temp;

	return _temp = _class = class GraphQueryComponent extends _react.Component {

		constructor(props, context) {
			super(props, context);
			this.state = {
				data: null,
				errors: null
			};
			this.uid = (0, _lodash.uniqueId)();

			this.request = () => {
				this.cancelRequest();
				const cancelablePromise = (0, _promises.makePromiseCancelable)(this.triggerGraphRequest());
				this.cancelRequestPromise = cancelablePromise.cancel;
				cancelablePromise.promise.then(results => {
					this.setState(results);
					this.cancelRequestPromise = false;
				}).catch(() => {}); // avoid console warnings
			};

			this.buildQuery(props);
		}

		componentDidMount() {
			const throttledRequest = (0, _lodash.throttle)(this.request, THROTTLE_DELAY, { leading: true });
			this.unsubscribe = this.context.graph.store.subscribe(throttledRequest);
			this.request();
		}

		componentWillUnmount() {
			this.cancelRequest();
			this.unsubscribe && this.unsubscribe();
			this.context.graph.store.dispatch((0, _actions.clearRequests)(this.uid));
		}

		componentWillReceiveProps(newProps) {
			this.buildQuery(newProps);
			this.request();
		}

		buildQuery(props) {
			if ((0, _lodash.isString)(mapPropsToQuery)) {
				this.query = mapPropsToQuery;
			} else {
				this.query = mapPropsToQuery(props);
			}
			this.variables = mapPropsToVariables(props);
			this.parsedQuery = (0, _quickGraphql.parse)(this.query, this.variables);
		}

		cancelRequest() {
			this.cancelRequestPromise && this.cancelRequestPromise();
		}

		triggerGraphRequest() {
			if (process.env.NODE_ENV === 'development') {
				return (0, _graphql.graphql)(this.context.graph.schema, this.query, this.context.graph.root, { uid: this.uid }, this.variables);
			}

			return (0, _quickGraphql.quickGraphql)(this.parsedQuery, this.context.graph.root, { uid: this.uid });
		}

		render() {
			return _react2.default.createElement(WrappedComponent, _extends({}, this.props, this.state));
		}
	}, _class.contextTypes = {
		graph: _react.PropTypes.object
	}, _temp;
};

exports.default = query;