'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _lodash = require('lodash');

const resolveNode = (node, resolver, context) => {
	let resolved = resolver;
	if ((0, _lodash.isFunction)(resolver)) {
		resolved = resolver(node.arguments, context);
	}
	if ((0, _lodash.isPlainObject)(resolved)) {
		return resolveNodes(node.nodes, resolved, context); // eslint-disable-line no-use-before-define
	}
	if ((0, _lodash.isArray)(resolved)) {
		return resolved.map(resolvedItem => resolveNodes(node.nodes, resolvedItem, context) // eslint-disable-line no-use-before-define
		);
	}
	return resolved;
};

const resolveNodes = (nodes, resolvers = {}, context) => {
	return nodes.reduce((memo, node) => {
		memo[node.name] = resolveNode(node, resolvers[node.name], context);
		return memo;
	}, {});
};

exports.default = (query, rootResolver, context) => new Promise(resolve => {
	const data = resolveNodes(query.nodes, rootResolver, context);
	resolve({ data });
});